/*
  MCS-DROS: Monitor and Control System - Data Recorder Operating Software
  Copyright (C) 2009-2010  Virginia Tech, Christopher Wolfe <chwolfe2@vt.edu>

  This file is part of MCS-DROS.

  MCS-DROS is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  MCS-DROS is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with MCS-DROS.  If not, see <http://www.gnu.org/licenses/>.
*/
#include <iostream>
#include <iomanip>
#include <stdlib.h>
#include <pthread.h>
#include "Message.h"
#include "Response.h"
#include "TimeKeeper.h"
#include "Socket.h"
#include <list>
#include <cstring>
#include <cstdio>

using namespace std;



void usage(string errorMsg){

	cout << "[Error]  " << errorMsg << endl;
	cout <<
	"Usage: " << endl <<
	"\tMsender " << endl <<
	"\t\t-Source \"XXX\" " << endl <<
	"\t\t-Destination \"YYY\" " << endl <<
	"\t\t-Type \"ZZZ\" " << endl <<
	"\t\t-ReferenceNumber 123456789 " << endl <<
	"\t\t-DestinationIpAddress aaa.bbb.ccc.ddd " << endl <<
	"\t\t-DestinationPort 12345 " << endl <<
	"\t\t-ResponseListenPort 12345 " << endl <<
	"\t\t[-Data \"string with no embedded nulls\"]" << endl << endl;
	exit(EXIT_FAILURE);
}
void parseResponse(Response* resp, string responseFormat){

}
int main(int argc, char * argv[]){
	size_t sendsize=0;
	size_t actuallySent=0;
	char * ptr=NULL;

	string msgSource;
	string msgDestination;
	string msgType;
	size_t msgReferenceNumber=0;
	size_t msgDataLength=0;
	size_t msgMJD;
	size_t msgMPM;
	char * msgData=NULL;

	unsigned short ResponseListenPort=0;
	unsigned short DestinationPort=0;
	string DestinationIpAddress;
	bool msgSourceSpecified=false;
	bool msgDestinationSpecified=false;
	bool msgTypeSpecified=false;
	bool msgReferenceNumberSpecified=false;
	bool DestinationIpAddressSpecified=false;
	bool DestinationPortSpecified=false;
	bool ResponseListenPortSpecified=false;
	bool msgDataSpecified=false; // the only one that does not have to be specified on the commandline
	bool verbose=false;



	string expectedMPM;
	string expectedMJD;
	string expectedSender;
	string expectedDestination;
	string expectedType;
	string expectedComment;
	string expectedStatus;
	string expectedAccepted;
	string expectedReference;

	bool expectedMPMSpecified=false;
	bool expectedMJDSpecified=false;
	bool expectedSenderSpecified=false;
	bool expectedDestinationSpecified=false;
	bool expectedTypeSpecified=false;
	bool expectedCommentSpecified=false;
	bool expectedStatusSpecified=false;
	bool expectedAcceptedSpecified=false;
	bool expectedReferenceSpecified=false;

	bool noListen=false;



	int i=1;
	int p=argc-1;
	while (i<argc){
		if 		 (strcmp(argv[i],"-Source")==0){
			if (i<p){	if (strlen(argv[i+1])<=3){		msgSource=string(argv[i+1]);		msgSourceSpecified=true;			i++;
			} else { usage("Invalid Source length. Must not be more than 3 characters"); }		} else { usage("Missing operand: Source"); }
		}else if (strcmp(argv[i],"-Destination")==0){
			if (i<p){	if (strlen(argv[i+1])<=3){		msgDestination=string(argv[i+1]);		msgDestinationSpecified=true;			i++;
			} else { usage("Invalid Destination length. Must not be more than 3 characters"); }		} else { usage("Missing operand: Destination"); }
		}else if (strcmp(argv[i],"-Type")==0){
			if (i<p){	if (strlen(argv[i+1])<=3){		msgType=string(argv[i+1]);		msgTypeSpecified=true;			i++;
			} else { usage("Invalid Type length. Must not be more than 3 characters"); }		} else { usage("Missing operand: Type"); }
		}else if (strcmp(argv[i],"-ReferenceNumber")==0){
			if (i<p){	if (strlen(argv[i+1])<=9){		msgReferenceNumber=strtoul(string(argv[i+1]).c_str(),NULL,10);		msgReferenceNumberSpecified=true;			i++;
			} else { usage("Invalid ReferenceNumber length. Must not be more than 9 characters"); }		} else { usage("Missing operand: ReferenceNumber"); }
		}else if (strcmp(argv[i],"-DestinationIpAddress")==0){
			if (i<p){	if (strlen(argv[i+1])<=15){		DestinationIpAddress=string(argv[i+1]);		DestinationIpAddressSpecified=true;			i++;
			} else { usage("Invalid Source length. Must not be more than 15 characters"); }		} else { usage("Missing operand: Source"); }
		}else if (strcmp(argv[i],"-DestinationPort")==0){
			if (i<p){	if (strlen(argv[i+1])<=5){		DestinationPort=(unsigned short)atoi(argv[i+1]);		DestinationPortSpecified=true;			i++;
			} else { usage("Invalid DestinationPort length. Must not be more than 5 characters"); }		} else { usage("Missing operand: DestinationPort"); }
		}else if (strcmp(argv[i],"-ResponseListenPort")==0){
			if (i<p){	if (strlen(argv[i+1])<=5){		ResponseListenPort=(unsigned short)atoi(argv[i+1]);		ResponseListenPortSpecified=true;			i++;
			} else { usage("Invalid ResponseListenPort length. Must not be more than 5 characters"); }		} else { usage("Missing operand: ResponseListenPort"); }
		}else if (strcmp(argv[i],"-Data")==0){
			if (i<p){
				if (strlen(argv[i+1])<=(8154)){
					msgData=argv[i+1];
					msgDataLength=strlen(argv[i+1]);
					msgDataSpecified=true;
					i++;
			} else { usage("Invalid Data length. Must not be more than 8154 characters"); }		} else { usage("Missing operand: Data"); }
		}else if (strcmp(argv[i],"-v")==0){
			verbose=true;
		}else if (strcmp(argv[i],"-nl")==0){
			noListen=true;
		}else if (strcmp(argv[i],"-expectedMPM")==0){ if (i<p){ 		expectedMPMSpecified=true; 			expectedMPM=argv[i+1]; 			i++;} else { usage("Missing operand: expectedMPM");}
		}else if (strcmp(argv[i],"-expectedMJD")==0){ if (i<p){ 		expectedMJDSpecified=true; 			expectedMJD=argv[i+1]; 			i++;} else { usage("Missing operand: expectedMJD");}
		}else if (strcmp(argv[i],"-expectedSender")==0){ if (i<p){ 		expectedSenderSpecified=true; 		expectedSender=argv[i+1]; 		i++;} else { usage("Missing operand: expectedSender");}
		}else if (strcmp(argv[i],"-expectedDestination")==0){ if (i<p){ expectedDestinationSpecified=true; 	expectedDestination=argv[i+1]; 	i++;} else { usage("Missing operand: expectedDestination");}
		}else if (strcmp(argv[i],"-expectedType")==0){ if (i<p){ 		expectedTypeSpecified=true; 		expectedType=argv[i+1]; 		i++;} else { usage("Missing operand: expectedType");}
		}else if (strcmp(argv[i],"-expectedComment")==0){ if (i<p){ 	expectedCommentSpecified=true; 		expectedComment=argv[i+1]; 		i++;} else { usage("Missing operand: expectedComment");}
		}else if (strcmp(argv[i],"-expectedStatus")==0){ if (i<p){ 		expectedStatusSpecified=true; 		expectedStatus=argv[i+1]; 		i++;} else { usage("Missing operand: expectedStatus");}
		}else if (strcmp(argv[i],"-expectedAccepted")==0){ if (i<p){ 	expectedAcceptedSpecified=true; 	expectedAccepted=argv[i+1]; 	i++;} else { usage("Missing operand: expectedAccepted");}
		}else if (strcmp(argv[i],"-expectedReference")==0){ if (i<p){ 	expectedReferenceSpecified=true; 	expectedReference=argv[i+1];	i++;} else { usage("Missing operand: expectedReference");}
		}else {
			usage(string("Unknown commandline option:")+argv[i]);
		}
		i++;
	}


	if (
		(! msgSourceSpecified) 				||
		(! msgDestinationSpecified) 		||
		(! msgTypeSpecified) 				||
		(! msgReferenceNumberSpecified)		||
		(! DestinationIpAddressSpecified) 	||
		(! DestinationPortSpecified) 		||
		(! ResponseListenPortSpecified) ){
		usage("You must specify all options except data and the verbosity flag");
	}


	Message* msg;
	msgMJD=TimeKeeper::getMJD();
	msgMPM=TimeKeeper::getMPM();
	if (!msgDataSpecified)
		msg = new Message(msgDestination, msgSource, msgType, msgReferenceNumber, 0, msgMJD, msgMPM, (char *) "");
	else{
		string mdata=msgData;
		size_t pos;
		size_t pos2;
		char MPM[8192];
		char MJD[8192];
		//cout << "STRING='"<<  mdata << "'" << endl;
		if ((pos = mdata.find("MPM"))!=string::npos){
			pos2 = mdata.find(" ",pos+1);
			string p = mdata.substr(pos+3,pos2-pos-3);
			int mpmDelta=atoi(p.c_str());
			if (mpmDelta >= 0){
				sprintf(MPM,"%09lu",msgMPM+(size_t)mpmDelta);
			} else {
				sprintf(MPM,"%09lu",msgMPM-(size_t)(mpmDelta*-1));
			}
			//cout << "Replacing 'MPM" << p << "' with '" << MPM << "'" << endl;
			mdata=mdata.replace(pos,pos2-pos,MPM);
		}
		//cout << "STRING='"<<  mdata << "'" << endl;
		if ((pos = mdata.find("MJD"))!=string::npos){
			pos2 = mdata.find(" ",pos+1);
			string p = mdata.substr(pos+3,pos2-pos-3);

			int mjdDelta=atoi(p.c_str());
			if (mjdDelta >= 0){
				sprintf(MJD,"%06lu",msgMJD+(size_t)mjdDelta);
			} else {
				sprintf(MJD,"%06lu",msgMJD-(size_t)(mjdDelta*-1));
			}
			//cout << "Replacing 'MJD" << p << "' with '" << MJD << "'" << endl;
			mdata=mdata.replace(pos,pos2-pos,MJD);
		}
		//cout << "STRING='"<<  mdata << "'" << endl;
		msgDataLength = mdata.length();
		msg = new Message(msgDestination, msgSource, msgType, msgReferenceNumber, msgDataLength, msgMJD, msgMPM, (char *)mdata.c_str());
	}
	if (msg==NULL) {
		cout << "allocation fault." << endl;
		exit (EXIT_FAILURE);
	}
	if (verbose) cout << *msg;
	Socket mySocket;
	mySocket.openInbound(ResponseListenPort);
	if (mySocket.inboundConnectionIsOpen() ){
		//cout << "Listen socket opened Successfully." << endl;
	} else {
		cout << "Could not open listen socket on port "<< ResponseListenPort << endl;
		exit (EXIT_FAILURE);
	}
	mySocket.openOutbound(DestinationIpAddress.c_str(), DestinationPort);
	if (mySocket.outboundConnectionIsOpen() ){
		//cout << "Send socket opened Successfully." << endl;
	} else {
		cout << "Could not open send socket to "<< DestinationIpAddress << " on port " << DestinationPort << endl;
		exit (EXIT_FAILURE);
	}
	ptr=msg->getMessageBlock(&sendsize);
	actuallySent=mySocket.send(ptr,sendsize);
	//cout  << endl << endl<< "Sent " <<  actuallySent << " bytes to " << DestinationIpAddress << " on port " << DestinationPort << endl << endl;
	// now receive response
	if(noListen) return 0;
	Response* response=NULL;
	char 		messageBuffer[MESSAGE_SIZE_LIMIT];
	size_t 		bytesReceived=0;
	bool error=false;
	while ( (mySocket.inboundConnectionIsOpen() && mySocket.outboundConnectionIsOpen()) && bytesReceived == 0){
		bytesReceived = mySocket.receive(messageBuffer, MESSAGE_SIZE_LIMIT);
		switch (bytesReceived){
			case 0:
				;//cout << "Zero-byte response, still waiting." << endl;
			break;
			default:
				if (bytesReceived>=MESSAGE_MINIMUM_SIZE){
					response=new Response(messageBuffer,bytesReceived);
					//cout << "received " <<  bytesReceived << " bytes" << endl;
					if (verbose)
						cout << *response;
					else
						cout << (*response).getComment() << endl;
					if ((expectedMPMSpecified) && (response->getMPM() != strtoul(expectedMPM.c_str(),NULL,10))) 						{error=true; printf ("[Error]  mismatched MPM              %lu != %lu\n",   response->getMPM() , strtoul(expectedMPM.c_str(),NULL,10));}
					if ((expectedMJDSpecified) && (response->getMJD() != strtoul(expectedMJD.c_str(),NULL,10))) 						{error=true; printf ("[Error]  mismatched MJD              %lu != %lu\n",   response->getMJD(), strtoul(expectedMJD.c_str(),NULL,10));}
					if ((expectedReferenceSpecified) && (response->getReferenceNumber() != strtoul(expectedReference.c_str(),NULL,10))) {error=true; printf ("[Error]  mismatched reference number %lu != %lu\n",   response->getReferenceNumber() , strtoul(expectedReference.c_str(),NULL,10));}
					if ((expectedSenderSpecified) && (response->getSender() != expectedSender)) 										{error=true; printf ("[Error]  mismatched Sender           '%s' != '%s'\n", response->getSender().c_str(), expectedSender.c_str());}
					if ((expectedDestinationSpecified) && (response->getDestination() != expectedDestination)) 							{error=true; printf ("[Error]  mismatched Destination      '%s' != '%s'\n", response->getDestination().c_str() , expectedDestination.c_str());}
					if ((expectedTypeSpecified) && (response->getType() != expectedType)) 												{error=true; printf ("[Error]  mismatched Type             '%s' != '%s'\n", response->getType().c_str(), expectedType.c_str());}
					if ((expectedStatusSpecified) && (response->getGeneralStatus() != expectedStatus)) 									{error=true; printf ("[Error]  mismatched Status           '%s' != '%s'\n", response->getGeneralStatus().c_str(), expectedStatus.c_str());}
					if ((expectedAcceptedSpecified) && (response->getAcceptReject() != expectedAccepted)) 								{error=true; printf ("[Error]  mismatched Accepted         '%s' != '%s'\n", response->getAcceptReject().c_str(), expectedAccepted.c_str());}
					if (expectedCommentSpecified){
						string com=response->getComment();
						char bufff[8192];
						strncpy(bufff, expectedComment.c_str(),8192);
						char* token = strtok(bufff," ");
						while (token){
							if (com.find(token)==string::npos){
								error=true;
								printf("[Error]  mismatched response data: couldn't find '%s' in '%s'\n",token,com.c_str());
							}
							token =strtok(NULL," ");
						}
					}
					if (expectedMPMSpecified || expectedMJDSpecified || expectedReferenceSpecified || expectedSenderSpecified || expectedDestinationSpecified || expectedTypeSpecified || expectedStatusSpecified || expectedAcceptedSpecified || expectedCommentSpecified){
						if (!error) printf("[PASS]\n"); else printf("[FAIL]\n");
					}
					delete (response);
				} else {
					cout << "[Error]  response was "<< bytesReceived << " bytes in length but the minimum is 38" << endl;
					exit (EXIT_FAILURE);
				}
			break;
		}
	}
}
